@@ -24,21 +24,21 @@ extern "C" {
  *
  * the version string like "1.2.3"
  */
-#define LIBEXSLT_DOTTED_VERSION "1.0.1"
+#define LIBEXSLT_DOTTED_VERSION "1.0.2"
 
 /**
  * LIBEXSLT_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBEXSLT_VERSION 200
+#define LIBEXSLT_VERSION 300
 
 /**
  * LIBEXSLT_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBEXSLT_VERSION_STRING "200"
+#define LIBEXSLT_VERSION_STRING "300"
 
 /**
  * ATTRIBUTE_UNUSED:
@@ -56,6 +56,23 @@ extern "C" {
 #define ATTRIBUTE_UNUSED
 #endif
 
+/**
+ * LIBEXSLT_PUBLIC:
+ *
+ * This macro is needed on Win32 when using MSVC. It enables the client code
+ * to access exported variables. It should expand to nothing when compiling
+ * this library itself, but must expand to __declspec(dllimport) when a
+ * client includes the library header and that only if it links dynamically
+ * against this library.
+ */
+#if !defined LIBEXSLT_PUBLIC
+#if defined _MSC_VER && !defined IN_LIBEXSLT && !defined LIBEXSLT_STATIC
+#define LIBEXSLT_PUBLIC __declspec(dllimport)
+#else
+#define LIBEXSLT_PUBLIC 
+#endif
+#endif
+
 #ifdef __cplusplus
 }
 #endif
