@@ -13,6 +13,13 @@
 #include "libxslt.h"
 
 #include <string.h>
+#ifdef HAVE_TIME_H
+#define __USE_XOPEN
+#include <time.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 
 #include <libxml/xmlmemory.h>
 #include <libxml/tree.h>
@@ -26,6 +33,7 @@
 #include "variables.h"
 #include "transform.h"
 #include "extra.h"
+#include "preproc.h"
 
 #ifdef WITH_XSLT_DEBUG
 #define WITH_XSLT_DEBUG_EXTRA
@@ -132,10 +140,14 @@ xsltFunctionNodeSet(xmlXPathParserContextPtr ctxt, int nargs){
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
     }
-    ctxt->value->type = XPATH_NODESET;
-    ctxt->value->boolval = 1;
+    if (ctxt->value->type == XPATH_XSLT_TREE) {
+	ctxt->value->type = XPATH_NODESET;
+    }
 }
 
+#if defined(HAVE_MKTIME) && defined(HAVE_LOCALTIME) && defined(HAVE_ASCTIME)
+#define WITH_LOCALTIME
+
 /**
  * xsltFunctionLocalTime:
  * @ctxt:  the XPath Parser context
@@ -145,48 +157,164 @@ xsltFunctionNodeSet(xmlXPathParserContextPtr ctxt, int nargs){
  *   string localTime(???)
  *
  * This function is available in Norm's extension namespace
+ * Code (and comments) contributed by Norm
  */
 static void
 xsltFunctionLocalTime(xmlXPathParserContextPtr ctxt, int nargs) {
-    if ((nargs < 0) || (nargs > 1)) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
-		"localTime() : invalid number of args %d\n", nargs);
-	ctxt->error = XPATH_INVALID_ARITY;
+    xmlXPathObjectPtr obj;
+    char *str;
+    char digits[5];
+    char result[29];
+    long int field;
+    time_t gmt, lmt;
+    struct tm gmt_tm;
+    struct tm *local_tm;
+ 
+    if (nargs != 1) {
+       xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
+       xsltGenericError(xsltGenericErrorContext,
+                      "localTime() : invalid number of args %d\n", nargs);
+       ctxt->error = XPATH_INVALID_ARITY;
+       return;
+    }
+ 
+    obj = valuePop(ctxt);
+
+    if (obj->type != XPATH_STRING) {
+	obj = xmlXPathConvertString(obj);
+    }
+    if (obj == NULL) {
+	valuePush(ctxt, xmlXPathNewString((const xmlChar *)""));
 	return;
     }
-    /* TODO : Norm's localTime() extension */
-    if (nargs == 1) {
-	xmlXPathStringFunction(ctxt, 1);
-    } else {
-	valuePush(ctxt, xmlXPathNewString((const xmlChar *)""));
-    }
+    
+    str = (char *) obj->stringval;
+
+    /* str = "$Date: 2001/08/14 16:46:18 $" */
+    memset(digits, 0, sizeof(digits));
+    strncpy(digits, str+7, 4);
+    field = strtol(digits, NULL, 10);
+    gmt_tm.tm_year = field - 1900;
+
+    memset(digits, 0, sizeof(digits));
+    strncpy(digits, str+12, 2);
+    field = strtol(digits, NULL, 10);
+    gmt_tm.tm_mon = field - 1;
+
+    memset(digits, 0, sizeof(digits));
+    strncpy(digits, str+15, 2);
+    field = strtol(digits, NULL, 10);
+    gmt_tm.tm_mday = field;
+
+    memset(digits, 0, sizeof(digits));
+    strncpy(digits, str+18, 2);
+    field = strtol(digits, NULL, 10);
+    gmt_tm.tm_hour = field;
+
+    memset(digits, 0, sizeof(digits));
+    strncpy(digits, str+21, 2);
+    field = strtol(digits, NULL, 10);
+    gmt_tm.tm_min = field;
+
+    memset(digits, 0, sizeof(digits));
+    strncpy(digits, str+24, 2);
+    field = strtol(digits, NULL, 10);
+    gmt_tm.tm_sec = field;
+
+    /* Now turn gmt_tm into a time. */
+    gmt = mktime(&gmt_tm);
+
+
+    /*
+     * FIXME: it's been too long since I did manual memory management.
+     * (I swore never to do it again.) Does this introduce a memory leak?
+     */
+    local_tm = localtime(&gmt);
+
+    /*
+     * Calling localtime() has the side-effect of setting timezone.
+     * After we know the timezone, we can adjust for it
+     */
+    lmt = gmt - timezone;
+
+    /*
+     * FIXME: it's been too long since I did manual memory management.
+     * (I swore never to do it again.) Does this introduce a memory leak?
+     */
+    local_tm = localtime(&lmt);
+
+    /*
+     * Now convert local_tm back into a string. This doesn't introduce
+     * a memory leak, so says asctime(3).
+     */
+
+    str = asctime(local_tm);           /* "Tue Jun 26 05:02:16 2001" */
+                                       /*  0123456789 123456789 123 */
+
+    memset(result, 0, sizeof(result)); /* "Thu, 26 Jun 2001" */
+                                       /*  0123456789 12345 */
+
+    strncpy(result, str, 20);
+    strcpy(result+20, "???");          /* tzname doesn't work, fake it */
+    strncpy(result+23, str+19, 5);
+
+    /* Ok, now result contains the string I want to send back. */
+    valuePush(ctxt, xmlXPathNewString((xmlChar *)result));
 }
+#endif
+
 
 /**
  * xsltRegisterExtras:
  * @ctxt:  a XSLT process context
  *
+ * Registers the built-in extensions. This function is deprecated, use
+ * xsltRegisterAllExtras instead.
+ */
+void
+xsltRegisterExtras(xsltTransformContextPtr ctxt ATTRIBUTE_UNUSED) {
+    xsltRegisterAllExtras();
+}
+
+/**
+ * xsltRegisterAllExtras:
+ *
  * Registers the built-in extensions
  */
 void
-xsltRegisterExtras(xsltTransformContextPtr ctxt) {
-    xsltRegisterExtFunction(ctxt, (const xmlChar *) "node-set",
-	                    XSLT_LIBXSLT_NAMESPACE, xsltFunctionNodeSet);
-    xsltRegisterExtFunction(ctxt, (const xmlChar *) "node-set",
-	                    XSLT_SAXON_NAMESPACE, xsltFunctionNodeSet);
-    xsltRegisterExtFunction(ctxt, (const xmlChar *) "node-set",
-	                    XSLT_XT_NAMESPACE, xsltFunctionNodeSet);
-    xsltRegisterExtFunction(ctxt, (const xmlChar *) "localTime",
-	                    XSLT_NORM_SAXON_NAMESPACE, xsltFunctionLocalTime);
-    xsltRegisterExtElement(ctxt, (const xmlChar *) "debug",
-	                    XSLT_LIBXSLT_NAMESPACE, xsltDebug);
-    xsltRegisterExtElement(ctxt, (const xmlChar *) "output",
-	                    XSLT_SAXON_NAMESPACE, xsltDocumentElem);
-    xsltRegisterExtElement(ctxt, (const xmlChar *) "write",
-	                    XSLT_XALAN_NAMESPACE, xsltDocumentElem);
-    xsltRegisterExtElement(ctxt, (const xmlChar *) "document",
-	                    XSLT_XT_NAMESPACE, xsltDocumentElem);
-    xsltRegisterExtElement(ctxt, (const xmlChar *) "document",
-	                    XSLT_NAMESPACE, xsltDocumentElem);
+xsltRegisterAllExtras (void) {
+    xsltRegisterExtModuleFunction((const xmlChar *) "node-set",
+				  XSLT_LIBXSLT_NAMESPACE,
+				  xsltFunctionNodeSet);
+    xsltRegisterExtModuleFunction((const xmlChar *) "node-set",
+				  XSLT_SAXON_NAMESPACE,
+				  xsltFunctionNodeSet);
+    xsltRegisterExtModuleFunction((const xmlChar *) "node-set",
+				  XSLT_XT_NAMESPACE,
+				  xsltFunctionNodeSet);
+#ifdef WITH_LOCALTIME
+    xsltRegisterExtModuleFunction((const xmlChar *) "localTime",
+				  XSLT_NORM_SAXON_NAMESPACE,
+				  xsltFunctionLocalTime);
+#endif
+    xsltRegisterExtModuleElement((const xmlChar *) "debug",
+				 XSLT_LIBXSLT_NAMESPACE,
+				 NULL,
+				 (xsltTransformFunction) xsltDebug);
+    xsltRegisterExtModuleElement((const xmlChar *) "output",
+				 XSLT_SAXON_NAMESPACE,
+				 xsltDocumentComp,
+				 (xsltTransformFunction) xsltDocumentElem);
+    xsltRegisterExtModuleElement((const xmlChar *) "write",
+				 XSLT_XALAN_NAMESPACE,
+				 xsltDocumentComp,
+				 (xsltTransformFunction) xsltDocumentElem);
+    xsltRegisterExtModuleElement((const xmlChar *) "document",
+				 XSLT_XT_NAMESPACE,
+				 xsltDocumentComp,
+				 (xsltTransformFunction) xsltDocumentElem);
+    xsltRegisterExtModuleElement((const xmlChar *) "document",
+				 XSLT_NAMESPACE,
+				 xsltDocumentComp,
+				 (xsltTransformFunction) xsltDocumentElem);
 }
