@@ -18,6 +18,21 @@
 extern "C" {
 #endif
 
+/**
+ * XSLT_REGISTER_FUNCTION_LOOKUP:
+ *
+ * registering macro, not general purpose at all but used in different modules
+ */
+#define XSLT_REGISTER_FUNCTION_LOOKUP(ctxt)			\
+    xmlXPathRegisterFuncLookup((ctxt)->xpathCtxt,		\
+	(void *) xsltXPathFunctionLookup,			\
+	(void *)(ctxt->xpathCtxt));
+
+xmlXPathFunction
+	xsltXPathFunctionLookup	(xmlXPathContextPtr ctxt,
+				 const xmlChar *name,
+				 const xmlChar *ns_uri);
+
 /*
  * Interfaces for the functions implementations
  */
