@@ -9,6 +9,12 @@
 #ifndef __XSLT_LIBXSLT_H__
 #define __XSLT_LIBXSLT_H__
 
+#ifdef WIN32
+#include <win32config.h>
+#else
+#include "config.h"
+#endif
+
 #if defined(WIN32) && !defined(__CYGWIN__)
 #include "xsltwin32config.h"
 #else
