@@ -29,6 +29,7 @@
 #include <libxml/xpathInternals.h>
 #include "xsltutils.h"
 #include "pattern.h"
+#include "templates.h"
 #include "numbersInternals.h"
 
 #ifndef FALSE
@@ -600,6 +601,15 @@ xsltNumberFormat(xsltTransformContextPtr ctxt,
     double number;
     xsltNumberFormatToken array[1024];
 
+    if ((data->format == NULL) && (data->has_format != 0)) {
+	data->format = xsltEvalAttrValueTemplate(ctxt, data->node,
+					     (const xmlChar *) "format",
+					     XSLT_NAMESPACE);
+    }
+    if (data->format == NULL) {
+	return;
+    }
+
     output = xmlBufferCreate();
     if (output == NULL)
 	goto XSLT_NUMBER_FORMAT_END;
@@ -1012,6 +1022,7 @@ xsltFormatNumberConversion(xsltDecimalFormatPtr self,
 
 OUTPUT_NUMBER:
     if (found_error != 0) {
+	xsltPrintErrorContext(NULL, NULL, NULL);
         xsltGenericError(xsltGenericErrorContext,
                 "xsltFormatNumberConversion : error in format string, using default\n");
 	default_sign = (number < 0.0) ? 1 : 0;
