@@ -380,6 +380,9 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		return(1);
             case XSLT_OP_ROOT:
 		if ((node->type == XML_DOCUMENT_NODE) ||
+#ifdef LIBXML_DOCB_ENABLED
+		    (node->type != XML_DOCB_DOCUMENT_NODE) ||
+#endif
 		    (node->type == XML_HTML_DOCUMENT_NODE))
 		    continue;
 		return(0);
@@ -402,9 +405,31 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			return(0);
 		}
 		continue;
-            case XSLT_OP_CHILD:
-		TODO /* Handle OP_CHILD */
+            case XSLT_OP_CHILD: {
+		xmlNodePtr lst;
+
+		if ((node->type != XML_ELEMENT_NODE) &&
+		    (node->type != XML_DOCUMENT_NODE) &&
+#ifdef LIBXML_DOCB_ENABLED
+		    (node->type != XML_DOCB_DOCUMENT_NODE) &&
+#endif
+		    (node->type != XML_HTML_DOCUMENT_NODE))
+		    return(0);
+
+		lst = node->children;
+
+		if (step->value != NULL) {
+		    while (lst != NULL) {
+			if ((lst->type == XML_ELEMENT_NODE) &&
+			    (xmlStrEqual(step->value, lst->name)))
+			    break;
+			lst = lst->next;
+		    }
+		    if (lst != NULL)
+			continue;
+		}
 		return(0);
+	    }
             case XSLT_OP_ATTR:
 		if (node->type != XML_ATTRIBUTE_NODE)
 		    return(0);
@@ -425,6 +450,9 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		}
 		continue;
             case XSLT_OP_PARENT:
+		if ((node->type != XML_ELEMENT_NODE) &&
+		    (node->type != XML_ATTRIBUTE_NODE))
+		    return(0);
 		node = node->parent;
 		if (node == NULL)
 		    return(0);
@@ -481,6 +509,9 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		/* TODO Handle IDs decently, must be done differently */
 		xmlAttrPtr id;
 
+		if (node->type != XML_ELEMENT_NODE)
+		    return(0);
+
 		id = xmlGetID(node->doc, step->value);
 		if ((id == NULL) || (id->parent != node))
 		    return(0);
@@ -502,7 +533,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		break;
 	    }
             case XSLT_OP_NS:
-		/* Namespace test */
+		if (node->type != XML_ELEMENT_NODE)
+		    return(0);
 		if (node->ns == NULL) {
 		    if (step->value != NULL)
 			return(0);
@@ -514,14 +546,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		}
 		break;
             case XSLT_OP_ALL:
-		switch (node->type) {
-		    case XML_DOCUMENT_NODE:
-		    case XML_HTML_DOCUMENT_NODE:
-		    case XML_ELEMENT_NODE:
-			break;
-		    default:
-			return(0);
-		}
+		if (node->type != XML_ELEMENT_NODE)
+		    return(0);
 		break;
 	    case XSLT_OP_PREDICATE: {
 		xmlNodePtr oldNode;
@@ -611,7 +637,8 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 			select->index = pos;
 			select->len = len;
 		    }
-		} else if ((select != NULL) && (select->op == XSLT_OP_ALL)) {
+		} else if ((select != NULL) && (select->op == XSLT_OP_ALL) &&
+			   (node->type == XML_ELEMENT_NODE)) {
 		    if ((select->previous != NULL) &&
 			(select->previous->parent == node->parent)) {
 			/*
@@ -860,7 +887,7 @@ xsltScanLiteral(xsltParserContextPtr ctxt) {
  * xsltScanName:
  * @ctxt:  the XPath Parser context
  *
- * [4] NameChar ::= Letter | Digit | '.' | '-' | '_' | ':' |
+ * [4] NameChar ::= Letter | Digit | '.' | '-' | '_' | 
  *                  CombiningChar | Extender
  *
  * [5] Name ::= (Letter | '_' | ':') (NameChar)*
@@ -883,7 +910,7 @@ xsltScanName(xsltParserContextPtr ctxt) {
 
     while ((IS_LETTER(NXT(len))) || (IS_DIGIT(NXT(len))) ||
            (NXT(len) == '.') || (NXT(len) == '-') ||
-	   (NXT(len) == '_') || (NXT(len) == ':') || 
+	   (NXT(len) == '_') || 
 	   (IS_COMBINING(NXT(len))) ||
 	   (IS_EXTENDER(NXT(len)))) {
 	buf[len] = NXT(len);
@@ -1208,60 +1235,91 @@ xsltCompileStepPattern(xsltParserContextPtr ctxt, xmlChar *token) {
 	    goto error;
     } else if (CUR == ':') {
 	NEXT;
-	if (NXT(1) != ':') {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
-		    "xsltCompileStepPattern : sequence '::' expected\n");
-	    ctxt->error = 1;
-	    goto error;
-	}
-	NEXT;
-	if (xmlStrEqual(token, (const xmlChar *) "child")) {
-	    name = xsltScanName(ctxt);
-	    if (name == NULL) {
+	if (CUR != ':') {
+	    xmlChar *prefix = token;
+	    xmlNsPtr ns;
+
+	    /*
+	     * This is a namespace match
+	     */
+	    token = xsltScanName(ctxt);
+	    ns = xmlSearchNs(ctxt->doc, ctxt->elem, prefix);
+	    if (ns == NULL) {
 		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
 		xsltGenericError(xsltGenericErrorContext,
-			"xsltCompileStepPattern : QName expected\n");
-		ctxt->error = 1;
-		goto error;
-	    }
-	    URI = xsltGetQNameURI(ctxt->elem, &token);
-	    if (token == NULL) {
+	    "xsltCompileStepPattern : no namespace bound to prefix %s\n",
+				 prefix);
 		ctxt->error = 1;
 		goto error;
 	    } else {
-		name = xmlStrdup(token);
-		if (URI != NULL)
-		    URL = xmlStrdup(URI);
+		URL = xmlStrdup(ns->href);
 	    }
-	    PUSH(XSLT_OP_CHILD, name, URL);
-	} else if (xmlStrEqual(token, (const xmlChar *) "attribute")) {
-	    name = xsltScanName(ctxt);
-	    if (name == NULL) {
-		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-		xsltGenericError(xsltGenericErrorContext,
-			"xsltCompileStepPattern : QName expected\n");
-		ctxt->error = 1;
-		goto error;
-	    }
-	    URI = xsltGetQNameURI(ctxt->elem, &token);
+	    xmlFree(prefix);
 	    if (token == NULL) {
-		ctxt->error = 1;
-		goto error;
+		if (CUR == '*') {
+		    NEXT;
+		    PUSH(XSLT_OP_NS, URL, NULL);
+		} else {
+		    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
+		    xsltGenericError(xsltGenericErrorContext,
+			    "xsltCompileStepPattern : Name expected\n");
+		    ctxt->error = 1;
+		    goto error;
+		}
 	    } else {
-		name = xmlStrdup(token);
-		if (URI != NULL)
-		    URL = xmlStrdup(URI);
+		PUSH(XSLT_OP_ELEM, token, URL);
 	    }
-	    PUSH(XSLT_OP_ATTR, name, URL);
 	} else {
-	    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
-	    xsltGenericError(xsltGenericErrorContext,
-		"xsltCompileStepPattern : 'child' or 'attribute' expected\n");
-	    ctxt->error = 1;
-	    goto error;
+	    NEXT;
+	    if (xmlStrEqual(token, (const xmlChar *) "child")) {
+		xmlFree(token);
+		token = xsltScanName(ctxt);
+		if (token == NULL) {
+		    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
+		    xsltGenericError(xsltGenericErrorContext,
+			    "xsltCompileStepPattern : QName expected\n");
+		    ctxt->error = 1;
+		    goto error;
+		}
+		URI = xsltGetQNameURI(ctxt->elem, &token);
+		if (token == NULL) {
+		    ctxt->error = 1;
+		    goto error;
+		} else {
+		    name = xmlStrdup(token);
+		    if (URI != NULL)
+			URL = xmlStrdup(URI);
+		}
+		PUSH(XSLT_OP_CHILD, name, URL);
+	    } else if (xmlStrEqual(token, (const xmlChar *) "attribute")) {
+		xmlFree(token);
+		token = xsltScanName(ctxt);
+		if (token == NULL) {
+		    xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
+		    xsltGenericError(xsltGenericErrorContext,
+			    "xsltCompileStepPattern : QName expected\n");
+		    ctxt->error = 1;
+		    goto error;
+		}
+		URI = xsltGetQNameURI(ctxt->elem, &token);
+		if (token == NULL) {
+		    ctxt->error = 1;
+		    goto error;
+		} else {
+		    name = xmlStrdup(token);
+		    if (URI != NULL)
+			URL = xmlStrdup(URI);
+		}
+		PUSH(XSLT_OP_ATTR, name, URL);
+	    } else {
+		xsltPrintErrorContext(NULL, NULL, NULL); /* TODO */
+		xsltGenericError(xsltGenericErrorContext,
+		    "xsltCompileStepPattern : 'child' or 'attribute' expected\n");
+		ctxt->error = 1;
+		goto error;
+	    }
+	    xmlFree(token);
 	}
-	xmlFree(token);
     } else if (CUR == '*') {
 	NEXT;
 	PUSH(XSLT_OP_ALL, token, NULL);
@@ -1631,8 +1689,7 @@ xsltAddTemplate(xsltStylesheetPtr style, xsltTemplatePtr cur,
         case XSLT_OP_CHILD:
         case XSLT_OP_PARENT:
         case XSLT_OP_ANCESTOR:
-        case XSLT_OP_NS:
-	    name = pat->steps[0].value;
+	    top = (xsltCompMatchPtr *) &(style->elemMatch);
 	    break;
         case XSLT_OP_ROOT:
 	    top = (xsltCompMatchPtr *) &(style->rootMatch);
@@ -1642,6 +1699,7 @@ xsltAddTemplate(xsltStylesheetPtr style, xsltTemplatePtr cur,
 	    break;
         case XSLT_OP_ID:
 	    /* TODO optimize ID !!! */
+        case XSLT_OP_NS:
         case XSLT_OP_ALL:
 	    top = (xsltCompMatchPtr *) &(style->elemMatch);
 	    break;
@@ -1885,6 +1943,23 @@ xsltGetTemplate(xsltTransformContextPtr ctxt, xmlNodePtr node,
 	    }
 	    list = list->next;
 	}
+	/*
+	 * Some of the tests for elements can also apply to documents
+	 */
+	if ((node->type == XML_DOCUMENT_NODE) ||
+	    (node->type == XML_HTML_DOCUMENT_NODE)) {
+	    list = curstyle->elemMatch;
+	    while ((list != NULL) &&
+		   ((ret == NULL)  || (list->priority > priority))) {
+		if (xsltTestCompMatch(ctxt, list, node,
+				      ctxt->mode, ctxt->modeURI)) {
+		    ret = list->template;
+		    break;
+		}
+		list = list->next;
+	    }
+	}
+
 	if (node->_private != NULL) {
 	    list = curstyle->keyMatch;
 	    while ((list != NULL) &&
