@@ -36,6 +36,7 @@
 #include "preproc.h"
 #include "extra.h"
 #include "imports.h"
+#include "extensions.h"
 
 #ifdef WITH_XSLT_DEBUG
 #define WITH_XSLT_DEBUG_PREPROC
@@ -75,39 +76,40 @@ xsltNewStylePreComp(xsltStylesheetPtr style, xsltStyleType type) {
     cur->type = type;
     switch (cur->type) {
         case XSLT_FUNC_COPY:
-            cur->func = xsltCopy;break;
+            cur->func = (xsltTransformFunction) xsltCopy;break;
         case XSLT_FUNC_SORT:
-            cur->func = xsltSort;break;
+            cur->func = (xsltTransformFunction) xsltSort;break;
         case XSLT_FUNC_TEXT:
-            cur->func = xsltText;break;
+            cur->func = (xsltTransformFunction) xsltText;break;
         case XSLT_FUNC_ELEMENT:
-            cur->func = xsltElement;break;
+            cur->func = (xsltTransformFunction) xsltElement;break;
         case XSLT_FUNC_ATTRIBUTE:
-            cur->func = xsltAttribute;break;
+            cur->func = (xsltTransformFunction) xsltAttribute;break;
         case XSLT_FUNC_COMMENT:
-            cur->func = xsltComment;break;
+            cur->func = (xsltTransformFunction) xsltComment;break;
         case XSLT_FUNC_PI:
-            cur->func = xsltProcessingInstruction;break;
+            cur->func = (xsltTransformFunction) xsltProcessingInstruction;
+	    break;
         case XSLT_FUNC_COPYOF:
-            cur->func = xsltCopyOf;break;
+            cur->func = (xsltTransformFunction) xsltCopyOf;break;
         case XSLT_FUNC_VALUEOF:
-            cur->func = xsltValueOf;break;
+            cur->func = (xsltTransformFunction) xsltValueOf;break;
         case XSLT_FUNC_NUMBER:
-            cur->func = xsltNumber;break;
+            cur->func = (xsltTransformFunction) xsltNumber;break;
         case XSLT_FUNC_APPLYIMPORTS:
-            cur->func = xsltApplyImports;break;
+            cur->func = (xsltTransformFunction) xsltApplyImports;break;
         case XSLT_FUNC_CALLTEMPLATE:
-            cur->func = xsltCallTemplate;break;
+            cur->func = (xsltTransformFunction) xsltCallTemplate;break;
         case XSLT_FUNC_APPLYTEMPLATES:
-            cur->func = xsltApplyTemplates;break;
+            cur->func = (xsltTransformFunction) xsltApplyTemplates;break;
         case XSLT_FUNC_CHOOSE:
-            cur->func = xsltChoose;break;
+            cur->func = (xsltTransformFunction) xsltChoose;break;
         case XSLT_FUNC_IF:
-            cur->func = xsltIf;break;
+            cur->func = (xsltTransformFunction) xsltIf;break;
         case XSLT_FUNC_FOREACH:
-            cur->func = xsltForEach;break;
+            cur->func = (xsltTransformFunction) xsltForEach;break;
         case XSLT_FUNC_DOCUMENT:
-            cur->func = xsltDocumentElem;break;
+            cur->func = (xsltTransformFunction) xsltDocumentElem;break;
 	case XSLT_FUNC_WITHPARAM:
 	    cur->func = NULL;break;
 	case XSLT_FUNC_PARAM:
@@ -125,7 +127,7 @@ xsltNewStylePreComp(xsltStylesheetPtr style, xsltStyleType type) {
 	}
     }
     cur->next = style->preComps;
-    style->preComps = cur;
+    style->preComps = (xsltElemPreCompPtr) cur;
 
     return(cur);
 }
@@ -194,8 +196,9 @@ xsltFreeStylePreComp(xsltStylePreCompPtr comp) {
  *
  * Pre process an XSLT-1.1 document element
  */
-static void
-xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst) {
+xsltElemPreCompPtr
+xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
+		 xsltTransformFunction function ATTRIBUTE_UNUSED) {
     xsltStylePreCompPtr comp;
     xmlChar *filename = NULL;
     xmlChar *base = NULL;
@@ -203,8 +206,7 @@ xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
     comp = xsltNewStylePreComp(style, XSLT_FUNC_DOCUMENT);
     if (comp == NULL)
-	return;
-    inst->_private = comp;
+	return (NULL);
     comp->inst = inst;
     comp->ver11 = 0;
 
@@ -273,6 +275,8 @@ error:
 	xmlFree(base);
     if (filename != NULL)
 	xmlFree(filename);
+
+    return ((xsltElemPreCompPtr) comp);
 }
 
 /************************************************************************
@@ -1205,14 +1209,17 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
  */
 void
 xsltFreeStylePreComps(xsltStylesheetPtr style) {
-    xsltStylePreCompPtr cur, next;
+    xsltElemPreCompPtr cur, next;
 
     if (style == NULL)
 	return;
     cur = style->preComps;
     while (cur != NULL) {
 	next = cur->next;
-	xsltFreeStylePreComp(cur);
+	if (cur->type == XSLT_FUNC_EXTENSION)
+	    cur->free(cur);
+	else
+	    xsltFreeStylePreComp((xsltStylePreCompPtr) cur);
 	cur = next;
     }
 }
@@ -1317,7 +1324,8 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    /* no computation needed */
 	    return;
 	} else if (IS_XSLT_NAME(inst, "document")) {
-	    xsltDocumentComp(style, inst);
+	    inst->_private = (void *) xsltDocumentComp(style, inst,
+				(xsltTransformFunction) xsltDocumentElem);
 	} else {
 	    xsltPrintErrorContext(NULL, style, inst);
 	    xsltGenericError(xsltGenericDebugContext,
@@ -1328,7 +1336,7 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	 * Add the namespace lookup here, this code can be shared by
 	 * all precomputations.
 	 */
-	cur = inst->_private;
+	cur = (xsltStylePreCompPtr) inst->_private;
 	if (cur != NULL) {
 	    int i = 0;
 
@@ -1340,14 +1348,14 @@ xsltStylePreCompute(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    cur->nsNr = i;
 	}
     } else {
-	if (IS_XSLT_NAME(inst, "document")) {
-	    xsltDocumentComp(style, inst);
-	} else {
-	    /*
-	     * Mark the element for later recognition.
-	     */
-	    if (inst->_private == NULL)
-		inst->_private = (void *) xsltExtMarker;
-	}
+	inst->_private =
+	    (void *) xsltPreComputeExtModuleElement(style, inst);
+
+	/*
+	 * Unknown element, maybe registered at the context
+	 * level. Mark it for later recognition.
+	 */
+	if (inst->_private == NULL)
+	    inst->_private = (void *) xsltExtMarker;
     }
 }
