@@ -38,6 +38,7 @@
 #include "documents.h"
 #include "extensions.h"
 #include "preproc.h"
+#include "extra.h"
 
 #ifdef WITH_XSLT_DEBUG
 #define WITH_XSLT_DEBUG_PARSING
@@ -119,6 +120,22 @@ PUSH_AND_POP(static, xmlChar *, exclPrefix)
  *									*
  ************************************************************************/
 
+/**
+ * xsltInit:
+ *
+ * Initializes the processor (e.g. registers built-in extensions,
+ * etc.)
+ */
+static void
+xsltInit (void) {
+    static int initialized = 0;
+
+    if (initialized == 0) {
+	initialized = 1;
+	xsltRegisterAllExtras();
+    }
+}
+
 /**
  * xsltIsBlank:
  * @str:  a string
@@ -328,6 +345,10 @@ xsltNewStylesheet(void) {
     cur->exclPrefixNr = 0;
     cur->exclPrefixMax = 0;
     cur->exclPrefixTab = NULL;
+    cur->extInfos = NULL;
+
+    xsltInit();
+
     return(cur);
 }
 
@@ -369,6 +390,7 @@ xsltFreeStylesheet(xsltStylesheetPtr sheet)
     xsltFreeNamespaceAliasHashes(sheet);
     xsltFreeStyleDocuments(sheet);
     xsltFreeStylePreComps(sheet);
+    xsltShutdownExts(sheet);
     if (sheet->doc != NULL)
         xmlFreeDoc(sheet->doc);
     if (sheet->variables != NULL)
@@ -1247,25 +1269,22 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 /**
  * xsltParseTemplateContent:
  * @style:  the XSLT stylesheet
- * @ret:  the "template" structure
- * @template:  the container node (can be a document for literal results)
+ * @templ:  the container node (can be a document for literal results)
  *
- * parse an XSLT template element content
+ * parse a template content-model
  * Clean-up the template content from unwanted ignorable blank nodes
  * and process xslt:text
  */
 
-static void
-xsltParseTemplateContent(xsltStylesheetPtr style, xsltTemplatePtr ret,
-	                 xmlNodePtr template) {
+void
+xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
     xmlNodePtr cur, delete;
     /*
      * This content comes from the stylesheet
      * For stylesheets, the set of whitespace-preserving
      * element names consists of just xsl:text.
      */
-    ret->elem = template;
-    cur = template->children;
+    cur = templ->children;
     delete = NULL;
     while (cur != NULL) {
 	if (delete != NULL) {
@@ -1363,7 +1382,7 @@ skip_children:
 	    cur = cur->parent;
 	    if (cur == NULL)
 		break;
-	    if (cur == template) {
+	    if (cur == templ) {
 		cur = NULL;
 		break;
 	    }
@@ -1386,7 +1405,7 @@ skip_children:
     /*
      * Skip the first params
      */
-    cur = template->children;
+    cur = templ->children;
     while (cur != NULL) {
 	if ((IS_XSLT_ELEM(cur)) && (!(IS_XSLT_NAME(cur, "param"))))
 	    break;
@@ -1412,8 +1431,6 @@ skip_children:
 	    break;
 	cur = cur->next;
     }
-
-    ret->content = template->children;
 }
 
 /**
@@ -1599,7 +1616,9 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     /*
      * parse the content and register the pattern
      */
-    xsltParseTemplateContent(style, ret, template);
+    xsltParseTemplateContent(style, template);
+    ret->elem = template;
+    ret->content = template->children;
     xsltAddTemplate(style, ret, mode, modeURI);
 
 error:
@@ -1652,28 +1671,44 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 
     cur = top->children;
 
+    /*
+     * process xsl:import elements
+     */
     while (cur != NULL) {
 	if (IS_BLANK_NODE(cur)) {
             cur = cur->next;
 	    continue;
 	}
-	if (!(IS_XSLT_ELEM(cur))) {
-#ifdef WITH_XSLT_DEBUG_PARSING
-	    xsltGenericDebug(xsltGenericDebugContext,
-		    "xsltParseStylesheetTop : found foreign element %s\n",
-		    cur->name);
-#endif
-            cur = cur->next;
-	    continue;
-	}
-	if (IS_XSLT_NAME(cur, "import")) {
+	if (IS_XSLT_ELEM(cur) && IS_XSLT_NAME(cur, "import")) {
 	    xsltParseStylesheetImport(style, cur);
 	} else
 	    break;
 	cur = cur->next;
     }
+    /*
+     * process other top-level elements
+     */
     while (cur != NULL) {
+	if (IS_BLANK_NODE(cur)) {
+	    cur = cur->next;
+	    continue;
+	}
+	if ((cur->type == XML_ELEMENT_NODE) && (cur->ns == NULL)) {
+	    xsltGenericError(xsltGenericErrorContext,
+		     "Found a top-level element %s with null namespace URI\n",
+		     cur->name);
+	    style->errors++;
+	    cur = cur->next;
+	    continue;
+	}
 	if (!(IS_XSLT_ELEM(cur))) {
+	    xsltTopLevelFunction function;
+
+	    function = xsltExtModuleTopLevelLookup(cur->name,
+						   cur->ns->href);
+	    if (function != NULL)
+		function(style, cur);
+
 #ifdef WITH_XSLT_DEBUG_PARSING
 	    xsltGenericDebug(xsltGenericDebugContext,
 		    "xsltParseStylesheetTop : found foreign element %s\n",
@@ -1819,7 +1854,9 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	/*
 	 * parse the content and register the pattern
 	 */
-	xsltParseTemplateContent(ret, template, (xmlNodePtr) doc);
+	xsltParseTemplateContent(ret, (xmlNodePtr) doc);
+	template->elem = (xmlNodePtr) doc;
+	template->content = doc->children;
 	xsltAddTemplate(ret, template, NULL, NULL);
     }
 
@@ -2108,4 +2145,3 @@ xsltLoadStylesheetPI(xmlDocPtr doc) {
     }
     return(ret);
 }
-
