@@ -21,21 +21,21 @@ extern "C" {
  *
  * the version string like "1.2.3"
  */
-#define LIBXSLT_DOTTED_VERSION "1.0.1"
+#define LIBXSLT_DOTTED_VERSION "1.0.2"
 
 /**
  * LIBXSLT_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXSLT_VERSION 10001
+#define LIBXSLT_VERSION 10002
 
 /**
  * LIBXSLT_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXSLT_VERSION_STRING "10001"
+#define LIBXSLT_VERSION_STRING "10002"
 
 /**
  * WITH_XSLT_DEBUG:
@@ -57,6 +57,23 @@ extern "C" {
 #define ATTRIBUTE_UNUSED
 #endif
 
+/**
+ * LIBXSLT_PUBLIC:
+ *
+ * This macro is needed on Win32 when using MSVC. It enables the client code
+ * to access exported variables. It should expand to nothing when compiling
+ * this library itself, but must expand to __declspec(dllimport) when a
+ * client includes the library header and that only if it links dynamically
+ * against this library.
+ */
+#if !defined LIBXSLT_PUBLIC
+#if defined _MSC_VER && !defined IN_LIBXSLT && !defined LIBXSLT_STATIC
+#define LIBXSLT_PUBLIC __declspec(dllimport)
+#else
+#define LIBXSLT_PUBLIC 
+#endif
+#endif
+
 #ifdef __cplusplus
 }
 #endif
