@@ -33,11 +33,14 @@
 #ifdef LIBXML_CATALOG_ENABLED
 #include <libxml/catalog.h>
 #endif
+#include <libxml/parserInternals.h>
+
 #include <libxslt/xslt.h>
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
 #include <libxslt/xsltutils.h>
 #include <libxslt/extensions.h>
+
 #include <libexslt/exsltconfig.h>
 
 #ifdef WIN32
@@ -50,7 +53,6 @@
 #include <sys/time.h>
 #endif /* WIN32 */
 
-extern int xmlLoadExtDtdDefaultValue;
 
 static int debug = 0;
 static int repeat = 0;
@@ -282,8 +284,8 @@ main(int argc, char **argv)
     defaultLoader = xmlGetExternalEntityLoader();
     xmlLineNumbersDefault(1);
 
-    if (novalid == 0)           /* TODO XML_DETECT_IDS | XML_COMPLETE_ATTRS */
-        xmlLoadExtDtdDefaultValue = 6;
+    if (novalid == 0)
+        xmlLoadExtDtdDefaultValue = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
     else
         xmlLoadExtDtdDefaultValue = 0;
     for (i = 1; i < argc; i++) {
@@ -406,9 +408,10 @@ main(int argc, char **argv)
     xmlSubstituteEntitiesDefault(1);
 
     /*
-     * Register the EXSLT extensions
+     * Register the EXSLT extensions and the test module
      */
     exsltRegisterAll();
+    xsltRegisterTestModule();
 
     for (i = 1; i < argc; i++) {
         if ((!strcmp(argv[i], "-maxdepth")) ||
@@ -504,7 +507,7 @@ main(int argc, char **argv)
         }
         xsltFreeStylesheet(cur);
     }
-    xsltUnregisterAllExtModules();
+    xsltCleanupGlobals();
     xmlCleanupParser();
     xmlMemoryDump();
     return (0);
